#ifndef GPMI_LIBS_MODULE_H
#define GPMI_LIBS_MODULE_H

struct gpmi_module {
	gpmi_library *library;                  //! Underlying library

	char *params;                           //! Params that need to be passed to the AI
	char *path;                             //! Module path - useful when the module wants to open the data dir located near the module binary

	gpmi_ProcInit        *proc_Init;        //! Proc to initialize function (Optional)
	gpmi_ProcUnInit      *proc_UnInit;      //! Proc to uninitialize function (Optional)
	gpmi_ProcRunTick     *proc_RunTick;     //! Proc to RunTick function (timer based applications)
	gpmi_ProcRunData     *proc_RunData;     //! Proc to RunData function (filter applications)
	gpmi_ProcDebug       *proc_Debug;       //! Proc to debugcommand functions (Optional)

	gpmi_global_event_t  proc_EventGlobal;  //! If not NULL, any event is passed here; preparsed call type
	gpmi_ProcEvent       **proc_EventArray; //! If proc_EventGlobal is not NULL, this and event_num are used
	int                  event_next_free;   //! used size of proc_EventArray
	int                  event_allocated;   //! allocated size of proc_EventArray

	void                 *mod_data;         //! module data: the module should store a pointer here to a strucutre where it stores its data
};

/* Functions available for public */
gpmi_module *gpmi_mod_load(const char *library_name, const char *params);
void gpmi_mod_unload(gpmi_module *module);

/* Functions to manipulate "current module" (thread safe) */
gpmi_module *gpmi_get_current_module(void);
void gpmi_set_current_module(gpmi_module *module);


/* Low level functions */
int gpmi_load_module(gpmi_module *module, const char *library_name);

#endif /* GPMI_LIBS_MODULE_H */
