#include "gpmi_scripts.h"
#include "scripts.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <gpmi/debug.h>

char *GetScriptDir()
{
	GPMI_Script_ClientData *MyData;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	return MyData->current_script_dir;
}

void gpmi_scripts_print_gpmi_error(gpmi_err_stack_t *entry, char *string)
{
	printf("\t%s\n", string);
}

int PkgLoad(char *pkg_name, int version)
{
	GPMI_Script_ClientData *MyData;
	gpmi_package *pkg;
	int res;
	int old_stack;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	gpmi_err_stack_destroy(NULL); /* XXX: We need a better solution, maybe marks in the stack */
	old_stack = gpmi_err_stack_enable();

	/* Load the package */
	res = gpmi_pkg_load(pkg_name, version, GPMI_Script_RegisterCommand, MyData, &pkg);

	/* Return error status */
	if (res < 0)
	{
		gpmi_debug(1, "Can't load package '%s'", pkg_name);
		gpmi_err_stack_process_str(gpmi_scripts_print_gpmi_error);
		return res;
	}

	if (old_stack == 0)
		gpmi_err_stack_disable();

	pkg->next = MyData->package;
	MyData->package = pkg;

	return res;
}

int Bind(int event, char *function)
{
	GPMI_Script_ClientData *MyData;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	/* Save the function-name */
	module_events_bind(MyData->events, event, strdup(function));

	return 0;
}

int Unbind(int event, char *function)
{
	GPMI_Script_ClientData *MyData;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	module_events_unbind(MyData->events, event, function, GPMI_Script_CompareModuleEventData, GPMI_Script_FreeModuleEventData);

	return 0;
}

int FakeEvent(int event, char *param)
{
	GPMI_Script_ClientData *MyData;
	GPMI_Script_EventData evdata;
	char **argv = NULL;
	int argc = 1;
	char tmp[1024];
	int i, last_i;
	char *p;
	char *last;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	/* Assume 1 param */
	argv = malloc(sizeof(char *));

	snprintf(tmp, sizeof(tmp), "%d", event);
	argv[0] = strdup(tmp);

	if (strlen(param) != 0) {
		p = strdup(param);
		last = p;
		last_i = 0;

		/* Count the params, and put them in seperate vars */
		for (i = 0; i < strlen(param) + 1; i++) {
			last_i++;
			if (p[i] == ' ' || p[i] == '\0') {
				p[i] = '\0';

				/* Increase param-size and fill data */
				/* XXX -- This is SLOW!! */
				argv = realloc(argv, sizeof(char *) * (argc + 1));
				argv[argc] = strdup(last);

				last += last_i;
				last_i = 0;
				argc++;
			}
		}

		free(p);
	}

	/* set up an event call structure */
	evdata.client_data = MyData;
	evdata.argv        = argv;
	evdata.argc        = argc;

	/* perform the call */
	module_events_call(MyData->events, event, GPMI_Script_Event_Callback, &evdata);

	/* Free the memory of the evdata */
	for (i = 0; i < argc; i++) {
		if (argv[i] != NULL)
			free(argv[i]);
	}


	return 0;
}

int Rehash()
{
	GPMI_Script_ClientData *MyData;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	if (MyData->info->rehash != NULL) {
		return MyData->info->rehash(MyData->client_data);
	} else {
		return 0;
	}
}

void *AddTimer(int TTL, int freq, char *script, char *user_data)
{
	GPMI_Script_ClientData *MyData;
	GPMI_Script_TimerData *td;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	/* Initialize timers if needed */
	if (MyData->timers == NULL) {
		if (gpmi_pkg_load("timer", 0, NULL, NULL, &(MyData->timer_pkg)) != 0) {
			gpmi_debug(0, "[ScriptPackage] Error: can't find timer package, TIMERS WON'T WORK, the script will probably fail.");
			return NULL;
		}
		MyData->timers = timer_init(GPMI_Script_Timer_Callback);
	}

	td = malloc(sizeof(GPMI_Script_TimerData));

	td->script    = strdup(script);
	td->user_data = strdup(user_data);
	/* Save the string, timer_callback will free it when timer expires */
	return timer_add(MyData->timers, MyData->last_frame, TTL, freq, td);
}

int DelTimer(void *timer_ptr)
{
	GPMI_Script_ClientData *MyData;
	gpmi_module *curr;

	curr = gpmi_get_current_module();
	MyData = curr->mod_data;

	/* Do nothing if we have no timers */
	if (MyData->timers == NULL)
		return -1;

	/* Save the string, timer_callback will free it when timer expires */
	return timer_del(MyData->timers, timer_ptr);
}

char *TimerInfo(void *td_, char *field)
{
	GPMI_Script_TimerData *td = td_; /* don't want to include the file with the structure in the .h containing the definition of this function */
	char *res;
	int len;

	if (td == NULL)
		return strdup("Invalid timer data");

	if (td->user_data != NULL)
		len = strlen(td->user_data) + 2;
	else
		len = 0;

	if (len < 128)
		len = 128;

	res = malloc(len);

	/* Ugly, but we can't depend on gpmi-extensions/strings :) */
	     if (strcasecmp(field, "ttl") == 0)       snprintf(res, len, "%d", td->TTL);
	else if (strcasecmp(field, "freq") == 0)      snprintf(res, len, "%d", td->freq);
	else if (strcasecmp(field, "user_data") == 0) strcpy(res, td->user_data);
	else strcpy(res, "Invalid field");

	return res;
}
