#ifndef TIMER_H
#define TIMER_H

#include <gpmi.h>

/**
 * The types that are used in TimerProc
 */
typedef enum timer_cmds {
	TIMER_INIT,
	TIMER_UNINIT,
	TIMER_TICK
} timer_cmds;

typedef struct timer {
	int frequency;            //! Frequency of the timer
	int next_frame;           //! The frame of the next call
	int TTL;                  //! Time To Live (in number of calls)
	void *data;
	struct timer *next;
} timer;

typedef void timer_callback(timer_cmds cmd, void *data, timer *timer);

typedef struct timer_list {
	timer_callback *CallBackProc;
	int next_frame;                   //! Next frame when any of our timers will trigger - optimization
	timer *FirstTimer;              //! The first timer struct in a linked list
} timer_list;


timer_list *timer_init(timer_callback *CallBackProc);                    		//! Inits a list
void timer_uninit(timer_list *TimerList);                                  //! Uninits a list
void timer_run(timer_list *TimerList, int frame_counter);                  //! run timers
timer *timer_add(timer_list *TimerList, int frmae_counter, int TTL, int Frequency, void *Data);
//!< Add a new timer to the list
int timer_del(timer_list *TimerList, timer *ptr);                        //! Del a Timer from a TimerList

#endif /* timer.h */
