#!/bin/sh

# I don't use /usr/bin/awk as interpreter because on different systems it may be
# located in different bin dirs.

# This file compares the output of a test (on the standard input) with a
# saved output known to be good (filename in $1). It also checks every equation
# and stops if any of them is broken.

awk -v "out_ok=$1" '

BEGIN {
	to_be_warned = 0
}

function warn_diff(line, first, S1) {
		print "Test failed: first word missmatch in line " line " when comparing to the saved output"
		print " (I will not warn for the same problem again, probalby it occurs for every line"
		print "  from now. Compare the files (out and out.OK) by hand. If the difference is"
		print "  because the example has been enhanced, and there is no problems with the"
		print "  output, just cp out out.OK)"
		print "   -> " first " != " S1

}

function add0x(v) {
	if (match(toupper(v), /^0[[:xdigit:]]+$/)) return "0x"v
	return v
}

#Check the "equations": the two neighbour words around a == must be equal
/ [!=]= / {
	gsub("\r", "")
	last = ""
	for (n=1; n<=NF; n++) {
		if ($n == "==") {
			m=n+1;
			if (add0x(last) != add0x($m)) {
				print "Test failed: equation is broken: " $0
				error++
			}
		}
		if ($n == "!=") {
			m=n+1;
			if (add0x(last) == add0x($m)) {
				print "Test failed: equation is broken: " $0
				error++
			}
		}

		last = $n
	}
}

#Check the first word of each line
{
	gsub("\r", "")
	if (out_ok == "")
		next


	first = $1
	getline < out_ok
	if (($1 != first) && (warned != 1)) {
		if (NR >= 4) {
			warn_diff(NR, first, $1)
		}
		else {
			to_be_warned   = NR
			to_be_warned_f = first
			to_be_warned_1 = $1
		}
		warned = 1
		error++
	}
}

END {
	if (NR < 4)  {
		print "Error: no output"
		exit  1
	}
	if (to_be_warned > 0) {
		warn_diff(to_be_warned, to_be_warned_first, to_be_warned_1)
	}
	if (error > 0) {
		print "Found errors, you have probably broken something"
		exit 1
	}
	else {
		print "OK. QC passed."
		exit 0
	}
}
'
